<?php
/*
Plugin Name: WP Hiero
Plugin URI: http://URI_Of_Page_Describing_Plugin_and_Updates
Description: A brief description of the Plugin.
Version: The Plugin's Version Number, e.g.: 1.0
Author: Jan Rozehnal
Author URI: http://URI_Of_The_Plugin_Author
License: A "Slug" license name e.g. GPL2
*/

include 'tables.php';
include 'defines.php';
include 'phonemes.php';

global $wh_prefabs, $wh_files;
global $wh_mode, $wh_scale, $wh_phonemes, $wh_text_conv;

$wh_mode    = WH_MODE_HTML; // default value
$wh_scale   = 100;          // default value

function WikiHieroSingle($glyph, $gardiner = true) {
    global $wh_prefabs;
    global $wh_files;
    global $wh_phonemes;
    global $wh_scale;
  if($gardiner = true)
  {
    return "<img style='margin:".WH_IMG_MARGIN."px;' src='/wp-content/plugins/wp-hiero/images/".WH_IMG_PRE.$glyph.".".WH_IMG_EXT."' title='".$glyph."' alt='".$glyph."' />";
  }  
  else
  {
    if(array_key_exists($glyph, $wh_phonemes))
  	{
  	  $code = $wh_phonemes[$glyph];
  	  if(array_key_exists($code, $wh_files))
  	  {
  		  return "<img style='margin:".WH_IMG_MARGIN."px;' $option src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$code}.".WH_IMG_EXT)."' title='".htmlspecialchars("{$code} [{$glyph}]")."' alt='".htmlspecialchars($glyph)."' />";
  		}
  	  else
  		{
        return "<font title='".htmlspecialchars($code)."'>".htmlspecialchars($glyph)."</font>";
  	 }
    }
  	else if(array_key_exists($glyph, $wh_files))
  	{
  	  return "<img style='margin:".WH_IMG_MARGIN."px;' $option src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$glyph}.".WH_IMG_EXT)."' title='".htmlspecialchars($glyph)."' alt='".htmlspecialchars($glyph)."' />";
  	}
    else
  	{
      return htmlspecialchars($glyph);
    }
  }
}

function WikiHieroHTML($hiero, $scale=WH_SCALE_DEFAULT, $line=false) {
    global $wh_prefabs;
    global $wh_files;
    global $wh_phonemes;
    global $wh_scale;

    if($scale != WH_SCALE_DEFAULT)
      $wh_scale = $scale;

    $html = "";

    if($line) {
      $html .= "<hr />\n";
	}

    //------------------------------------------------------------------------
    // Split text into block, then split block into item
    $block = array();
    $block[0] = array();
    $block[0][0] = "";
    $block_id = 0;
    $item_id = 0;
    $parenthesis = 0;
    $type = WH_TYPE_NONE;
    $is_cartouche = false;
    $is_striped = false;

	for($char=0; $char<strlen($hiero); $char++) {

		if( $hiero[$char] == '(' ) {
			$parenthesis++;
		} else if( $hiero[$char] == ')' ) {
			$parenthesis--;
		}

		if( $parenthesis == 0 ) {
			if($hiero[$char] == '-' || $hiero[$char] == ' ') {
				if($type != WH_TYPE_NONE) {
					$block_id++;
					$block[$block_id] = array();
					$item_id = 0;
					$block[$block_id][$item_id] = "";
					$type = WH_TYPE_NONE;
				}
			}
		} else {// don't slit block if inside parenthesis
			if($hiero[$char] == '-') {
				$item_id++;
				$block[$block_id][$item_id] = '-';
				$type = WH_TYPE_CODE;
			}
		}

		if($hiero[$char] == '!' ) {
			if($item_id > 0) {
				$block_id++;
				$block[$block_id] = array();
				$item_id = 0;
			}
			$block[$block_id][$item_id] = $hiero[$char];
			$type = WH_TYPE_END;

		} else if(ereg("[*:()]", $hiero[$char])) {

			if($type == WH_TYPE_GLYPH || $type == WH_TYPE_CODE) {
				$item_id++;
				$block[$block_id][$item_id] = "";
			}
		$block[$block_id][$item_id] = $hiero[$char];
		$type = WH_TYPE_CODE;

		} else if(ctype_alnum($hiero[$char]) || $hiero[$char] == '.' || $hiero[$char] == '<' || $hiero[$char] == '>') {
			if($type == WH_TYPE_END) {
				$block_id++;
				$block[$block_id] = array();
				$item_id = 0;
				$block[$block_id][$item_id] = "";
			} else if($type == WH_TYPE_CODE) {
				$item_id++;
				$block[$block_id][$item_id] = "";
			}
			$block[$block_id][$item_id] .= $hiero[$char];
			$type = WH_TYPE_GLYPH;
		}
    }

	// DEBUG: See the block split table
	if(WH_DEBUG_MODE) {

		foreach($block as $code) {
			echo "| ";
			foreach($code as $item) {
				echo "$item | ";
			}
			echo "<br />\n";
		}
	}

	$contentHtml = $tableHtml = $tableContentHtml = "";

	foreach($block as $code) {

		// simplest case, the block contain only 1 code -> render
		if(count($code) == 1)
		{
			if($code[0] == "!") { // end of line 
				$tableHtml = "</tr>".WH_TABLE_E.WH_TABLE_S."<tr>\n";
				if($line) {
					$contentHtml .= "<hr />\n";
				}

			} else if(strchr($code[0], '<')) { // start cartouche
				$contentHtml .= WH_TD_S.WH_RenderGlyph($code[0]).WH_TD_E;
				$is_cartouche = true;
				$contentHtml .= "<td>".WH_TABLE_S."<tr><td height='".intval(WH_CARTOUCHE_WIDTH * $wh_scale / 100)."px' class='cartouche_top' bgcolor='black'></td></tr><tr><td>".WH_TABLE_S."<tr>";

			} else if(strchr($code[0], '>')) { // end cartouche
				$contentHtml .= "</tr>".WH_TABLE_E."</td></tr><tr><td height='".intval(WH_CARTOUCHE_WIDTH * $wh_scale / 100)."px' class='cartouche_bottom' bgcolor='black'>&nbsp;</td></tr>".WH_TABLE_E."</td>";
				$is_cartouche = false;
				$contentHtml .= WH_TD_S.WH_RenderGlyph($code[0]).WH_TD_E;

			} else if($code[0] != "") { // assum is glyph or '..' or '.'
				$option = "height='".WH_Resize($code[0], $is_cartouche)."px'";

				$contentHtml .= WH_TD_S.WH_RenderGlyph($code[0], $option).WH_TD_E;
			}

		// block contain more than 1 glyph
		} else {

			// convert all code into '&' to test prefabs glyph
			$temp = "";
			foreach($code as $t) {
				if(ereg("[*:!()]", $t[0])) {
					$temp .= "&";
				} else {
					$temp .= $t;
				}
			}

        // test is block is into tje prefabs list
        if(in_array($temp, $wh_prefabs)) {
			$option = "height='".WH_Resize($temp, $is_cartouche)."px'";

			$contentHtml .= WH_TD_S.WH_RenderGlyph($temp, $option).WH_TD_E;

        // block must be manualy computed
        } else {
			// get block total height
			$line_max = 0;
			$total    = 0;
			$height   = 0;

			foreach($code as $t) {
				if($t == ":") {
					if($height > $line_max) {
						$line_max = $height;
					}
					$total += $line_max;
					$line_max = 0;

				} else if($t == "*") {
					if($height > $line_max) {
						$line_max = $height;
					}
				} else {
					if(array_key_exists($t, $wh_phonemes)) {
						$glyph = $wh_phonemes[$t];
					} else {
						$glyph = $t;
					}
					if(array_key_exists($glyph, $wh_files)) {
						$height = 2 + $wh_files[$glyph][1];
					}
				}
          	} // end foreach

			if($height > $line_max) {
				$line_max = $height;
			}

			$total += $line_max;

			// render all glyph into the block
			$temp = "";
			foreach($code as $t) {

				if($t == ":") {
					$temp .= "<br />";

				} elseif($t == "*") {
					$temp .= " ";

				} else {
					// resize the glyph according to the block total height
					$option = "height='".WH_Resize($t, $is_cartouche, $total)."px'";
					$temp .= WH_RenderGlyph($t, $option);
				}
			} // end foreach

			$contentHtml .= WH_TD_S.$temp.WH_TD_E;
		}
		$contentHtml .= "\n";
		}

		if(strlen($contentHtml) > 0) {
			$tableContentHtml .= $tableHtml.$contentHtml;
			$contentHtml = $tableHtml = "";
		}
	}

	if(strlen($tableContentHtml) > 0) {
		$html .= WH_TABLE_S."<tr>\n".$tableContentHtml."</tr>".WH_TABLE_E;
	}

	return "<table style='display:inline;' class='mw-hierotable'><tr><td>\n$html\n</td></tr></table>";
}


function WH_Resize($item, $is_cartouche=false, $total=0) {
	global $wh_phonemes;
	global $wh_files;
	global $wh_scale;
  
	if(array_key_exists($item, $wh_phonemes)) {
		$glyph = $wh_phonemes[$item];
	} else {
	  $glyph = $item;
	}

	$margin = 2 * WH_IMG_MARGIN;
	if($is_cartouche) {
		$margin += 2 * intval(WH_CARTOUCHE_WIDTH * $wh_scale / 100);
	}

	if(array_key_exists($glyph, $wh_files)) {
		$height = $margin + $wh_files[$glyph][1];
		if($total) {
			if($total > WH_HEIGHT) {
				return (intval( $height * WH_HEIGHT / $total ) - $margin) * $wh_scale / 100;
			} else {
				return ($height - $margin) * $wh_scale / 100;
			}
		} else {
			if($height > WH_HEIGHT) {
				return (intval( WH_HEIGHT * WH_HEIGHT / $height ) - $margin) * $wh_scale / 100;
			} else {
				return ($height - $margin) * $wh_scale / 100;
			}
		}
	}

	return (WH_HEIGHT - $margin) * $wh_scale / 100;
 }


function WH_RenderGlyph($glyph, $option='') {
	global $wh_mode;
	global $wh_phonemes;
	global $wh_files;
	global $wh_scale;

	if($glyph == "..") { // Render void block
	  $width = WH_HEIGHT;
	  return "<table width='{$width}px' border='0' cellspacing='0' cellpadding='0'><tr><td>&nbsp;</td></tr></table>";
	}
	else if($glyph == ".") // Render half-width void block
	{
	  $width = WH_HEIGHT/2;
	  return "<table width='{$width}px' border='0' cellspacing='0' cellpadding='0'><tr><td>&nbsp;</td></tr></table>";
	}
	else if($glyph == '<') // Render open cartouche
	{
	  $height = intval(WH_HEIGHT * $wh_scale / 100);
	  $code = $wh_phonemes[$glyph];
	  return "<img src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$code}.".WH_IMG_EXT)."' height='{$height}px' title='".htmlspecialchars($glyph)."' alt='".htmlspecialchars($glyph)."' />";
	}
	else if($glyph == '>') // Render close cartouche
	{
	  $height = intval(WH_HEIGHT * $wh_scale / 100);
	  $code = $wh_phonemes[$glyph];
	  return "<img src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$code}.".WH_IMG_EXT)."' height='{$height}px' title='".htmlspecialchars($glyph)."' alt='".htmlspecialchars($glyph)."' />";
	}

	if(array_key_exists($glyph, $wh_phonemes))
	{
	  $code = $wh_phonemes[$glyph];
	  if(array_key_exists($code, $wh_files))
		return "<img style='margin:".WH_IMG_MARGIN."px;' $option src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$code}.".WH_IMG_EXT)."' title='".htmlspecialchars("{$code} [{$glyph}]")."' alt='".htmlspecialchars($glyph)."' />";
	  else
		return "<font title='".htmlspecialchars($code)."'>".htmlspecialchars($glyph)."</font>";
	}
	else if(array_key_exists($glyph, $wh_files))
	  return "<img style='margin:".WH_IMG_MARGIN."px;' $option src='".htmlspecialchars(WH_IMG_DIR.WH_IMG_PRE."{$glyph}.".WH_IMG_EXT)."' title='".htmlspecialchars($glyph)."' alt='".htmlspecialchars($glyph)."' />";
	else
	  return htmlspecialchars($glyph);
} 

function single_hiero_shortcode( $atts, $content = null ) {
  if(trim($content) != "")
  {
    return WikiHieroSingle(htmlspecialchars_decode($content));
  }
}

add_shortcode( 'single_hiero', 'single_hiero_shortcode' );

function hiero_shortcode( $atts, $content = null ) {
   if(trim($content) != "")
   {
   
    return WikiHieroHTML(htmlspecialchars_decode($content));
  }
}

add_shortcode( 'hiero', 'hiero_shortcode' );

?>
